package SerialHeat;

public class DataArray {
	
	public static int X=900;
	public static int Y=600;
	
	public static double[][] currArr;
	public static double[][] nextArr;
	public static boolean[][] isHeater;
	public static boolean[][] isInsulator;
	public static boolean[][] isRed;
	public static boolean[][] isBlack;
	public static int brushSize = 3;
		
	public DataArray() {
		currArr= new double[X][Y];
		nextArr = new double[X][Y];
		isHeater = new boolean[X][Y];
		isInsulator = new boolean[X][Y];
		isRed = new boolean[X][Y];
		isBlack = new boolean[X][Y];
	}
		
	public static void insertSquare(int sx, int sy, int ex, int ey) {
		if (sx<0) {
			throw new IllegalArgumentException("Invalid x coordinate");
		} else if(sy<0) {
			throw new IllegalArgumentException("Invalid y coordinate");
		}	
		if (ex > X) {
			throw new IllegalArgumentException("Out of bounds!");
		} else if (ey > Y) {
			throw new IllegalArgumentException("Out of bounds!");
		}
		for (int i=sx;i<ex;i++) {
			for (int j=sy;j<ey;j++) {
				currArr[i][j] = 999;
			}
		}
	}
	
	public static void clear() {
		for (int i=0;i<X;i++) {
			for (int j=0;j<Y;j++) {
				currArr[i][j] = 0;
				isInsulator[i][j] = false;
				isHeater[i][j] = false;
			}
		}
	}
		
	public static void insertHeater(int sx, int sy, int ex, int ey) {
		if (sx<0) {
			throw new IllegalArgumentException("Invalid x coordinate");
		} else if(sy<0) {
			throw new IllegalArgumentException("Invalid y coordinate");
		}
			
		if (ex > X) {
			throw new IllegalArgumentException("Out of bounds!");
		} else if (ey > Y) {
			throw new IllegalArgumentException("Out of bounds!");
		}
		
		for (int i=sx;i<ex;i++) {
			for (int j=sy;j<ey;j++) {
				isHeater[i][j] = true;
				currArr[i][j] = 999;
			}
		}
	}
	
	public static void insertInsulator(int sx, int sy, int ex, int ey) {	
		if (sx<0) {
			throw new IllegalArgumentException("Invalid x coordinate");
		} else if(sy<0) {
			throw new IllegalArgumentException("Invalid y coordinate");
		}	
		
		if (ex > X) {
			throw new IllegalArgumentException("Out of bounds!");
		} else if (ey > Y) {
			throw new IllegalArgumentException("Out of bounds!");
		}
		
		for (int i=sx;i<ex;i++) {
			for (int j=sy;j<ey;j++) {
				isInsulator[i][j] = true;
			}
		}
	}
	
	public static void diffuse() {		
		for (int x=0;x<X;x++) {
			for (int y=0;y<Y;y++){
				if (!isInsulator[x][y]) {
					double currRed;
					if (isHeater[x][y]) {
						currRed = 999;
					} else {
						currRed = currArr[x][y];
					}
					double ownRed = currRed/4;
					double giveRed = (currRed-ownRed)/8;
					if (x==0) {
						if (y==0) { //corner
							nextArr[x][y] +=ownRed;
							if (isInsulator[x+1][y]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y] +=giveRed;
							}
							
							if (isInsulator[x+1][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y+1] +=giveRed;
							}
							
							if (isInsulator[x][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y+1] +=giveRed;
							}
						} else if (y==(Y-1)) {//corner
							nextArr[x][y] +=ownRed;
							
							if (isInsulator[x][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y-1] +=giveRed;
							}
							
							if (isInsulator[x+1][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y-1] +=giveRed;
							}
							
							if (isInsulator[x+1][y]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y] +=giveRed;
							}
						} else { //left edge
							nextArr[x][y] +=ownRed;
							
							if (isInsulator[x][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y-1] +=giveRed;
							}
							
							if (isInsulator[x+1][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y-1] +=giveRed;
							}
							if (isInsulator[x+1][y]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y] +=giveRed;
							}
							
							if (isInsulator[x+1][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x+1][y+1] +=giveRed;
							}
							
							if (isInsulator[x][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y+1] +=giveRed;
							}
						}
					} else if (x==(X-1)) {
						if (y==0) { //corner
							nextArr[x][y] +=ownRed;
							
							if (isInsulator[x-1][y]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y] +=giveRed;
							}
							
							if (isInsulator[x-1][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y+1] +=giveRed;
							}
							
							if (isInsulator[x][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y+1] +=giveRed;
							}
						} else if (y==(Y-1)) {//corner
							nextArr[x][y] +=ownRed;
							
							if (isInsulator[x][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y-1] +=giveRed;
							}
							
							if (isInsulator[x-1][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y-1] +=giveRed;
							}
							
							if (isInsulator[x-1][y]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y] +=giveRed;
							}
						} else { //right edge
							nextArr[x][y] +=ownRed;
							
							if (isInsulator[x][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y-1] +=giveRed;
							}
							
							if (isInsulator[x-1][y-1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y-1] +=giveRed;
							}
							
							if (isInsulator[x-1][y]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y] +=giveRed;
							}
							
							if (isInsulator[x-1][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x-1][y+1] +=giveRed;
							}
							if (isInsulator[x][y+1]) {
								nextArr[x][y] +=giveRed;
							} else {
								nextArr[x][y+1] +=giveRed;
							}
						}
					} else if (y==0) {
						//top edge
						nextArr[x][y] +=ownRed;
						
						if (isInsulator[x-1][y]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y] +=giveRed;
						}
						
						if (isInsulator[x-1][y+1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y+1] +=giveRed;
						}

						if (isInsulator[x][y+1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x][y+1] +=giveRed;
						}
						
						if (isInsulator[x+1][y+1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y+1] +=giveRed;
						}
						
						if (isInsulator[x+1][y]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y] +=giveRed;
						}
					} else if (y==(Y-1)) {
					 //bottom edge
						nextArr[x][y] +=ownRed;
						
						if (isInsulator[x-1][y]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y] +=giveRed;
						}
						
						if (isInsulator[x-1][y-1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y-1] +=giveRed;
						}
						
						if (isInsulator[x][y-1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x][y-1] +=giveRed;
						}
						
						if (isInsulator[x+1][y-1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y-1] +=giveRed;
						}
						
						if (isInsulator[x+1][y]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y] +=giveRed;
						}
					
					} else { //standard pixel in the middle
						nextArr[x][y] +=ownRed;

						if (isInsulator[x][y+1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x][y+1] +=giveRed;
						}
						
						if (isInsulator[x][y-1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x][y-1] +=giveRed;
						}
						
						if (isInsulator[x+1][y]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y] +=giveRed;
						}
						
						if (isInsulator[x+1][y-1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y-1] +=giveRed;
						}
						
						if (isInsulator[x+1][y+1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x+1][y+1] +=giveRed;
						}
						
						if (isInsulator[x-1][y]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y] +=giveRed;
						}
						
						if (isInsulator[x-1][y-1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y-1] +=giveRed;
						}
						
						if (isInsulator[x-1][y+1]) {
							nextArr[x][y] +=giveRed;
						} else {
							nextArr[x-1][y+1] +=giveRed;
						}
					}
				}
			}
		}
	}
		
	public void setA(double[][] arr) {
		currArr = arr;
	}
	
	public static void printArr(boolean t) {
		if (t) {
			for (int y=0;y<Y;y++){	
				for (int x=0;x<X;x++) {
					if (currArr[x][y]<10) {
						System.out.printf("  %.3f ",currArr[x][y]);
					} else if(currArr[x][y]<100) {
						System.out.printf(" %.3f ",currArr[x][y]);
					} else {
						System.out.printf("%.3f ",currArr[x][y]);
					}
				}
				System.out.println();
			}
		}else {
			for (int y=0;y<Y;y++){	
				for (int x=0;x<X;x++) {
					if (nextArr[x][y]<10) {
						System.out.printf("  %.3f ",nextArr[x][y]);
					} else if(nextArr[x][y]<100) {
						System.out.printf(" %.3f ",nextArr[x][y]);
					} else {
						System.out.printf("%.3f ",nextArr[x][y]);
					}
				}
				System.out.println();
			}
		}
	}

	public static void drawHotSpot(int x, int y) {
		for (int i=x-brushSize;i<x+brushSize;i++) {
			int k=i;
			if (i<0)
				k=0;
			if (i>=DataArray.X)
				k = DataArray.X-1;
			for (int j=y-brushSize;j<y+brushSize;j++) {
				int l=j;
				if (j<0)
					l=0;
				if (j>=DataArray.Y)
					l = DataArray.Y-1;
				currArr[k][l] = 999;
			}
		}
	}
	
	public static void drawInsulator(int x, int y) {
		for (int i=x-brushSize;i<x+brushSize;i++) {
			int k =i;
			if (i<0)
				k=0;
			if (i>=DataArray.X)
				k = DataArray.X-1;
			for (int j=y-brushSize;j<y+brushSize;j++) {
				int l=j;
				if (j<0)
					l=0;
				if (j>=DataArray.Y)
					l = DataArray.Y-1;
				isInsulator[k][l] = true;
			}
		}
	}
	
	public static void drawHeater(int x, int y) {
		for (int i=x-brushSize;i<x+brushSize;i++) {
			int k=i;
			if (i<0)
				k=0;
			if (i>=DataArray.X)
				k = DataArray.X-1;
			for (int j=y-brushSize;j<y+brushSize;j++) {
				int l = j;
				if (j<0)
					l=0;
				if (j>=DataArray.Y)
					l = DataArray.Y-1;
				isHeater[k][l] = true;
				currArr[k][l] = 999;
			}
		}
	}
	
	public static void setRed(int x, int y) {
		for (int i=x-brushSize;i<x+brushSize;i++) {
			int k = i;
			if (i<0)
				k=0;
			if (i>=DataArray.X)
				k = DataArray.X-1;
			for (int j=y-brushSize;j<y+brushSize;j++) {
				int l = j;
				if (j<0)
					l=0;
				if (j>=DataArray.Y)
					l = DataArray.Y-1;
				isRed[k][l] = true;
			}
		}
	}		
	
	public static void setBlack(int x, int y) {
		for (int i=x-brushSize;i<x+brushSize;i++) {
			int k = i;
			if (i<0)
				k=0;
			if (i>=DataArray.X)
				k = DataArray.X-1;
			for (int j=y-brushSize;j<y+brushSize;j++) {
				int l = j;
				if (j<0)
					l=0;
				if (j>=DataArray.Y)
					l = DataArray.Y-1;
				isBlack[k][l] = true;
			}
		}
	}		
}


